#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>

const char* ssid = <wifi_goes_here>;
const char* password = <wifi_password>;

const char* mqtt_server = <mqtt_host>;

const char* mqtt_topic = "sensor/bedoccupancy";

#define Bed0Pin 5
#define Bed1Pin 4

volatile byte interruptCounter = 0;

WiFiClient wifi_client;
PubSubClient mqtt_client(wifi_client);

void setup() {
  // put your setup code here, to run once:
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.begin(115200);
  setup_wifi();
  setup_mqtt();

  
  pinMode(Bed0Pin, INPUT_PULLUP);
  pinMode(Bed1Pin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(Bed0Pin), handleInterrupt, CHANGE);
  attachInterrupt(digitalPinToInterrupt(Bed1Pin), handleInterrupt, CHANGE);
  digitalWrite(LED_BUILTIN, HIGH);
}

void setup_mqtt() {
  mqtt_client.setServer(mqtt_server, 1883);
}

void setup_wifi() {
  delay(10);
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  digitalWrite(LED_BUILTIN, LOW);
  delay(100);
  digitalWrite(LED_BUILTIN, HIGH);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
  digitalWrite(LED_BUILTIN, LOW);
  delay(100);
  digitalWrite(LED_BUILTIN, HIGH);
  delay(400);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void reconnect() {
  while(!mqtt_client.connected()) {
    Serial.print("Attempting MQTT connection...");
    if (mqtt_client.connect("ESP8266Client")) {
      Serial.println("connected.");
    } else {
      Serial.print("failed, rc=");
      Serial.print(mqtt_client.state());
      Serial.println(" try again in 5 seconds");
      digitalWrite(LED_BUILTIN, LOW);
      delay(100);
      digitalWrite(LED_BUILTIN, HIGH);
      delay(5000);
    }
  }
}

void handleInterrupt() {
  ++interruptCounter;
}

void publish(int val0, int val1) {
  if (!mqtt_client.connected()) {
    reconnect();
  }
  mqtt_client.loop();
  
  StaticJsonBuffer<200> jsonBuffer;
  JsonObject& root = jsonBuffer.createObject();
  root["sensor"] = "bed";
  JsonArray& data = root.createNestedArray("data");
  data.add(val0);
  data.add(val1);
  root.printTo(Serial);
  Serial.println();
  String json;
  root.printTo(json);
  
  mqtt_client.publish(mqtt_topic, json.c_str(), true);
}

void loop() {
  // put your main code here, to run repeatedly:
  if (interruptCounter>0) {
    interruptCounter = 0;
    digitalWrite(LED_BUILTIN, LOW);
    auto val0 = digitalRead(Bed0Pin);
    auto val1 = digitalRead(Bed1Pin);
    publish(val0, val1);
    delay(100);
    digitalWrite(LED_BUILTIN, HIGH);
  }
}
